#!/usr/bin/env python

'''

    @version $Id: AlarmPanel.py,v 1.7 2005/01/18 05:06:23 davidfancella Exp $

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

from wxPython.wx import *
from pyAlarm import paGlobals
from pyAlarm.GUI import AlarmDialog
from pyAlarm.data.alarmlistitem import alarmListItem
import string

ID_NEW      = 101
ID_OPEN     = 102
ID_SAVE     = 103
ID_SAVEAS   = 104
ID_EXIT     = 105
ID_CUT      = 201
ID_COPY     = 202
ID_PASTE    = 203
ID_STARTCLOCK = 204
ID_STOPCLOCK = 205

ID_ABOUT    = 301

ID_ALARMLIST= 401

ID_ALARMNEW = 501
ID_ALARMEDIT= 502
ID_ALARMDEL = 503

ID_VIEWPLAY = 601
ID_PLAYALL  = 602

ID_PLAY     = 10001
ID_STOP     = 10002

class AlarmPanel(wxPanel):
    SelectedAlarms = []
    days = ['monday','tuesday','wednesday','thursday','friday','saturday','sunday']
    def __init__(self, *args, **kwds):
        # begin wxGlade: MainFrame.__init__
        kwds["style"] = wxDEFAULT_FRAME_STYLE
        wxPanel.__init__(self, *args, **kwds)
        self.AlarmList = wxListCtrl(self, ID_ALARMLIST, size = (530,250), style=wxLC_REPORT)
        '''self.AlarmList.InsertColumn(0,"Name",wxLIST_FORMAT_LEFT,240)
        self.AlarmList.InsertColumn(1,"M",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(2,"T",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(3,"W",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(4,"T",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(5,"F",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(6,"S",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(7,"S",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(8,"Time",wxLIST_FORMAT_CENTRE, 200)'''
        self.Edit = wxMenu()
        self.Edit.Append(ID_ALARMNEW, "&New Alarm", "Create New Alarm")
        self.Edit.Append(ID_ALARMEDIT, "&Edit Alarm", "Edit Selected Alarm")
        self.Edit.Append(ID_ALARMDEL, "&Delete Alarm", "Delete Selected Alarm")

        self.__set_properties()
        self.__do_layout()
        # end wxGlade

        EVT_MENU(self, ID_ALARMNEW, self.CreateNewAlarm)
        EVT_MENU(self, ID_ALARMEDIT, self.EditAlarm)
        EVT_MENU(self, ID_ALARMDEL, self.DeleteAlarm)
        
        EVT_LIST_ITEM_RIGHT_CLICK(self, ID_ALARMLIST, self.OnAlarmRightClick)
        EVT_LIST_ITEM_SELECTED(self, ID_ALARMLIST, self.OnAlarmselect)
        EVT_LIST_ITEM_DESELECTED(self, ID_ALARMLIST, self.OnAlarmDeSelect)

        self.RecreateList()

    def __set_properties(self):
        pass
        
    def __do_layout(self):
        # begin wxGlade: MainFrame.__do_layout
        self.sizer = wxBoxSizer(wxVERTICAL)
        self.sizer.Add(self.AlarmList, 1, wxALL|wxEXPAND, 0)
        self.SetAutoLayout(1)
        self.SetSizer(self.sizer)
        self.Layout()
        self.Centre()
        # end wxGlade

    def OnNew(self, event):       
        self.RecreateList()
        paGlobals.gCommand.NewProject()

    def OnAlarmRightClick(self, event):
        self.PopupMenu(self.Edit, event.GetPoint())

    def OnAlarmselect(self, event):
        self.SelectedAlarms.append(event.GetIndex())

    def OnAlarmDeSelect(self, event):
        self.SelectedAlarms.remove(event.GetIndex())

    def CreateNewAlarm(self, event):
        self.SelectedAlarms = [] # Clear selected alarms, otherwise errors will result
        dlg = AlarmDialog.AlarmDialog(self,-1,"Test")
        if(dlg.ShowModal()==wxID_OK):
            tmpitem = self.__ProcessAlarmDialog(dlg)
            tmpitemagain = paGlobals.gCommand.alarms.AddNew(tmpitem)
            self.AddAlarmListItem(-1, tmpitemagain)
        dlg.Destroy()

    def __ProcessAlarmDialog(self, dlg):
        tmpitem = alarmListItem()
        tmpitem.SetName(dlg.AlarmNameText.GetValue())
        tmpitem.SetDay('monday',dlg.monday.GetValue())
        tmpitem.SetDay('tuesday',dlg.tuesday.GetValue())
        tmpitem.SetDay('wednesday',dlg.wednesday.GetValue())
        tmpitem.SetDay('thursday',dlg.thursday.GetValue())
        tmpitem.SetDay('friday',dlg.friday.GetValue())
        tmpitem.SetDay('saturday',dlg.saturday.GetValue())
        tmpitem.SetDay('sunday',dlg.sunday.GetValue())
        tmphour = str(dlg.hours.GetValue())
        tmpminute = str(dlg.minutes.GetValue())
        ampm = dlg.ampm.GetStringSelection()
        tmptime = wxDateTime()
        tmptime.ParseTime(tmphour + ':' + tmpminute + ' ' + ampm)
        tmpitem.SetTime(tmptime)
        return tmpitem
        
    def AddAlarmListItem(self, index, newItem):
        if(index == -1): index = self.AlarmList.GetItemCount() # pass -1 for index
                                                               # to create a new item
        self.AlarmList.InsertStringItem(index,newItem.GetName())
        for a in range(len(self.days)):
            if(newItem.GetDay(self.days[a])):
                self.tempAlarmCap = "X"
            else:
                self.tempAlarmCap = ""
            self.AlarmList.SetStringItem(index,a+1,self.tempAlarmCap)
        self.AlarmList.SetStringItem(index,8,newItem.GetTime().FormatTime())
        self.AlarmList.SetItemData(index,newItem.index)
        
    def RecreateList(self):
        self.AlarmList.ClearAll()
        self.AlarmList.InsertColumn(0,"Name",wxLIST_FORMAT_LEFT,240)
        self.AlarmList.InsertColumn(1,"M",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(2,"T",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(3,"W",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(4,"T",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(5,"F",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(6,"S",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(7,"S",wxLIST_FORMAT_CENTRE,30)
        self.AlarmList.InsertColumn(8,"Time",wxLIST_FORMAT_CENTRE,100)
        self.AlarmList.Hide()
        self.mycounter = 0
        for i in paGlobals.gCommand.alarms.alarms:
            self.AddAlarmListItem(-1,i)
        self.AlarmList.Show()

    def EditAlarm(self, event):
        if(self.AlarmList.GetSelectedItemCount() == 0):
            dlg = wxMessageDialog(self, "Please Select an Alarm to Edit\n",
                              "No Alarm Selected!", wxOK | wxICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return
        dlg = AlarmDialog.AlarmDialog(NULL,-1,"Test")
        tmpsomething = self.AlarmList.GetItemData(self.SelectedAlarms[0])
        tempAlarmEdit = paGlobals.gCommand.alarms.GetItem(tmpsomething)
        dlg.uniqueID = tempAlarmEdit.index
        dlg.AlarmNameText.SetValue(tempAlarmEdit.GetName())
        dlg.monday.SetValue(tempAlarmEdit.GetDay('monday'))
        dlg.tuesday.SetValue(tempAlarmEdit.GetDay('tuesday'))
        dlg.wednesday.SetValue(tempAlarmEdit.GetDay('wednesday'))
        dlg.thursday.SetValue(tempAlarmEdit.GetDay('thursday'))
        dlg.friday.SetValue(tempAlarmEdit.GetDay('friday'))
        dlg.saturday.SetValue(tempAlarmEdit.GetDay('saturday'))
        dlg.sunday.SetValue(tempAlarmEdit.GetDay('sunday'))
        tmpHour = tempAlarmEdit.GetTime().GetHour()
        if tmpHour == 12:
            ampm = 'PM'
        elif tmpHour == 0:
            tmpHour = 12
            ampm = 'AM'
        elif tmpHour > 12:
            ampm = 'PM'
            tmpHour = tmpHour - 12
        else:
            ampm = 'AM'
        dlg.hours.SetValue(tmpHour)
        dlg.minutes.SetValue(tempAlarmEdit.GetTime().GetMinute())
        dlg.ampm.SetStringSelection(ampm)
        dlg.setWindowTitle("Edit Alarm")
        if(dlg.ShowModal()==wxID_OK):
            tmpitem = self.__ProcessAlarmDialog(dlg)
            paGlobals.gCommand.alarms.Modify(dlg.uniqueID,tmpitem)
            editdays = paGlobals.gCommand.alarms.GetItem(dlg.uniqueID)
            self.AddAlarmListItem(self.SelectedAlarms[0],editdays)
            self.AlarmList.DeleteItem(self.SelectedAlarms[0])
        dlg.Destroy()
        self.SelectedAlarms.pop(0)
        self.RecreateList()

    def DeleteAlarm(self, event):
        self.alarmtodelete = []
        self.alarmlistitemtodelete = []
        # First gather up the alarms that need to be deleted.
        for a in self.SelectedAlarms:
            self.alarmtodelete.append(self.AlarmList.GetItemData(a))
            self.alarmlistitemtodelete.append(a)
        # delete the alarms from the data structure
        for a in self.alarmtodelete:
            print a
            paGlobals.gCommand.alarms.Delete(a)
        # delete the alarms from the widget
        for a in self.alarmlistitemtodelete:
            self.AlarmList.DeleteItem(a)
        self.SelectedAlarms = []

# end of class AlarmPanel