#!/usr/bin/env python

'''

    @version $Id: PlayList.py,v 1.13 2005/01/18 03:39:07 davidfancella Exp $

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

from wxPython.wx import *
from pyAlarm import paGlobals
import dircache
import os
from pyAlarm.events import *

ID_FILENEW      = 2001
ID_FILEOPEN     = 2002
ID_FILESAVE     = 2003
ID_FILESAVEAS   = 2004
ID_FILECLOSE    = 2005

ID_LIST_ADD     = 3001
ID_LIST_DELETE  = 3002
ID_LIST_ADDDIR  = 3003

ID_PL_LISTBOX   = 3101

subdirlist = []


class PlayListPanel(wxPanel):
    currentFile = ""
    openedFile = 0
    selectedItems = []
    def __init__(self, *args, **kwds):
        # begin wxGlade: AlarmPlayList.__init__
        kwds["style"] = wxICONIZE|wxCAPTION|wxMINIMIZE|wxMINIMIZE_BOX|wxMAXIMIZE|wxMAXIMIZE_BOX|wxSYSTEM_MENU|wxRESIZE_BORDER
        wxPanel.__init__(self, *args, **kwds)
        self.playlist = paGlobals.gCommand.GetFiler()
        self.PlayListView = wxListCtrl(self, ID_PL_LISTBOX, size = (530,25), style=wxLC_REPORT)
        self.PlayListView.InsertColumn(0,"Artist",wxLIST_FORMAT_LEFT,240)

        # Menu Bar
        self.MasterMenu = wxMenu()
        self.File = wxMenu()
        self.File.Append(ID_FILENEW,"&New","Create PlayList")
        self.File.Append(ID_FILEOPEN,"&Open","Open Existing PlayList")
        self.File.Append(ID_FILESAVE,"&Save","Save PlayList")
        self.File.Append(ID_FILESAVEAS,"Save &As...","Save PlayList As...")
        self.MasterMenu.AppendMenu(100000, "File", self.File)
        self.Edit = wxMenu()
        self.Edit.Append(ID_LIST_ADD, "&Add Song", "Add Song to PlayList")
        self.Edit.Append(ID_LIST_ADDDIR, "Add &Directory", "Add Directory to Playlist")
        self.Edit.Append(ID_LIST_DELETE, "&Delete Song", "Delete Selected Songs")
        self.MasterMenu.AppendMenu(100001, "Edit", self.Edit)
        # Menu Bar end

        self.__set_properties()
        self.__do_layout()
        # end wxGlade

        EVT_MENU(self, ID_FILENEW, self.OnFileNew)
        EVT_MENU(self, ID_FILEOPEN, self.OnFileOpen)
        EVT_MENU(self, ID_FILESAVE, self.OnFileSave)
        EVT_MENU(self, ID_FILESAVEAS, self.OnFileSaveAs)
        EVT_MENU(self, ID_LIST_ADD, self.OnPlayListAdd)
        EVT_MENU(self, ID_LIST_ADDDIR, self.OnPlayListAddDir)
        EVT_MENU(self, ID_LIST_DELETE, self.OnPlayListDelete)
        EVT_CONTEXT_MENU(self, self.OnPLRightClick)
        #EVT_LIST_ITEM_RIGHT_CLICK(self, ID_PL_LISTBOX, self.OnPLRightClick)
        EVT_LIST_ITEM_SELECTED(self, ID_PL_LISTBOX, self.OnPLselect)
        EVT_LIST_ITEM_DESELECTED(self, ID_PL_LISTBOX, self.OnPLDeSelect)
        #EVT_LISTBOX(self, ID_PL_LISTBOX, self.OnSelect)
        #EVT_LISTBOX_DCLICK(self, ID_PL_LISTBOX, self.OnPlaySong)
        
        self.RecreateView()

    def __set_properties(self):
        # begin wxGlade: AlarmPlayList.__set_properties
        self.SetTitle("PlayList Editor")
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: AlarmPlayList.__do_layout
        playListSizer = wxBoxSizer(wxVERTICAL)
        playListSizer.Add(self.PlayListView, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 0)
        self.SetAutoLayout(1)
        self.SetSizer(playListSizer)
        playListSizer.Fit(self)
        playListSizer.SetSizeHints(self)
        self.Layout()
        # end wxGlade

    def OnFileNew(self, event):
        self.playlist.ClearList()
        self.PlayListView.ClearAll()
        paGlobals.gCommand.openedPlayList = 0

    def OnFileOpen(self, event):
        self.dlg = wxFileDialog(NULL, "Open...","","","*",wxOPEN)
        self.dlg.SetFilename(self.currentFile)
        if(self.dlg.ShowModal()==wxID_OK):
            self.currentFile = self.dlg.GetPath()
            self.OpenPlayList(self.currentFile)
        else:
            self.dlg.Destroy()
            return
        self.dlg.Destroy()
        
    def OnPLselect(self, event):
        self.selectedItems.append(event.GetIndex())

    def OnPLDeSelect(self, event):
        self.selectedItems.remove(event.GetIndex())
        
    def OnPLRightClick(self, event):
        pos = self.ScreenToClient(event.GetPosition())
        self.PopupMenuXY(self.MasterMenu, pos.x/2, pos.y/2)
        
    def OpenPlayList(self, filename):
        paGlobals.gCommand.OpenFilerFile(filename)
        self.RecreateView()
        paGlobals.gCommand.openPlayList = 1
        paGlobals.gCommand.config.Write("/Playlist/OpenFile",filename)
        self.GetParent().openPlayList = 1

    def OnFileSave(self, event):
        if(paGlobals.gCommand.openPlayList == 1):
            self.SaveFile(self.currentFile)

    def SaveFile(self, filename):
        paGlobals.gCommand.SaveFilerFile(filename)
        paGlobals.gCommand.config.Write("/Playlist/OpenFile",filename)
        self.GetParent().openPlayList = 1

    def OnFileSaveAs(self, event):
        self.dlg = wxFileDialog(NULL, "Save As...","","","*",wxSAVE|wxOVERWRITE_PROMPT)
        self.dlg.SetFilename(self.currentFile)
        if(self.dlg.ShowModal()==wxID_OK):
            self.currentFile = self.dlg.GetPath()
        else:
            self.dlg.Destroy()
            return
        self.dlg.Destroy()
        self.SaveFile(self.currentFile)
        #AlarmAll.openedPlayList = 1

    def RecreateView(self):
        self.PlayListView.ClearAll()
        self.PlayListView.InsertColumn(0,"Artist",wxLIST_FORMAT_LEFT,240)
        self.PlayListView.Hide()
        for a in paGlobals.gCommand.GetPlaylist():
            self.PlayListView.InsertStringItem(self.PlayListView.GetItemCount(),a)
        self.PlayListView.Show()

    def OnPlayListAdd(self, event):
        self.dlg = wxFileDialog(NULL, "Select File","","","*",wxOPEN)
        if(self.dlg.ShowModal()==wxID_OK):
            paGlobals.gCommand.AddPLFile(self.dlg.GetPath())
            self.AddFile(-1, self.dlg.GetPath())
        else:
            self.dlg.Destroy()
            return
        self.dlg.Destroy()

    def AddFile(self, index, path):
        if(index == -1): index = self.PlayListView.GetItemCount() # pass -1 for index
                                                               # to create a new item
                                                               
        paGlobals.gCommand.AddPLFile(path)
        
        self.PlayListView.InsertStringItem(index, os.path.basename(path))
        
    def OnPlayListAddDir(self, event):
        self.dlg = wxDirDialog(self)
        if(self.dlg.ShowModal()==wxID_OK):
            paGlobals.gCommand.ScanPLDir(self.dlg.GetPath())
        self.dlg.Destroy()
        self.RecreateView()

    def OnPlayListDelete(self, event):
        self.alarmtodelete = []
        self.alarmlistitemtodelete = []
        # First gather up the songs that need to be deleted.
        for a in self.selectedItems:
            self.alarmtodelete.append(self.PlayListView.GetItemData(a))
            self.alarmlistitemtodelete.append(a)
        # delete the alarms from the data structure
        for a in self.alarmtodelete:
            paGlobals.gCommand.playlistdata.Delete(a)
        self.selectedItems = []
        self.RecreateView()

    def AddTags(self, tags):
        title = tags.Artist + " - " + tags.Title
        if title != " - ":
            try:
                self.PlayListView.SetStringItem(tags.Index,0,title)
            except:
                pass
        paGlobals.gCommand.SetTags(tags)
        
# end of class AlarmPlayList


