#!/usr/bin/env python

'''
File Information:
    
   Last changed by: $Author: davidfancella $
                On: $Date: 2005/01/18 06:59:39 $
                
   This file is part of $Name:  $
   
   $Revision: 1.25 $
   
'''


'''

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

from wxPython.wx import *
import time
from pyAlarm.GUI import AlarmPanel
from pyAlarm.GUI import PlayList
from pyAlarm.GUI import mixerpanel
from pyAlarm.GUI import paabout
from pyAlarm.GUI import preferences
from pyAlarm import graphics
import cStringIO
from pyAlarm import command
from pyAlarm import paGlobals
from pyAlarm.events import *
import os

ID_NEW      = 101
ID_OPEN     = 102
ID_SAVE     = 103
ID_SAVEAS   = 104
ID_EXIT     = 105
ID_CUT      = 201
ID_COPY     = 202
ID_PASTE    = 203

ID_OPENPLAYLIST = 250
ID_SAVEASPLAYLIST = 251

ID_ABOUT    = 301

ID_ALARMLIST= 401

ID_ALARMNEW = 501
ID_ALARMEDIT= 502
ID_ALARMDEL = 503

ID_VIEWPLAY = 601
ID_PLAYALL  = 602

ID_PLAY     = 10001
ID_STOP     = 10002
ID_SNOOZE   = 10003
ID_PAUSE    = 10004
ID_PREVIOUS = 10005
ID_NEXT     = 10006
ID_SLEEP    = 10007

ID_PREFS    = 10030

class pyAlarmApp(wxApp):
    def OnInit(self):
        wxInitAllImageHandlers()
        paGlobals.gCommand = command.paCommand()        
        frame = MainFrame(NULL, -1, "pyAlarm", pos=paGlobals.gCommand.mainPos)
        paGlobals.gCommand.Init(frame)
        paGlobals.gCommand.OpenPLStored()
        frame.PlaylistPanel.RecreateView()
        frame.Show(true)
        self.SetTopWindow(frame)
        return true

    def OnExit(self):
        return true

class MainFrame(wxFrame):
    SelectedAlarms = []
    currentFile = "MyAlarms.alarm"
    isalarmplaying = 0
    openAlarms = 0
    openPlayList = 0
    def __init__(self, *args, **kwds):
        # begin wxGlade: MainFrame.__init__
        kwds["style"] = wxDEFAULT_FRAME_STYLE
        wxFrame.__init__(self, *args, **kwds)
        
        try:
             self.SetIcon(graphics.getApplIcon())
        finally:
            pass
        
        self.MainNotebook = wxNotebook(self, -1, style=wxNB_RIGHT)
        self.AlarmPanel = AlarmPanel.AlarmPanel(self.MainNotebook, -1)
        self.PlaylistPanel = PlayList.PlayListPanel(self.MainNotebook, -1)
        self.Mixer = mixerpanel.MixerPanel(self.MainNotebook, -1)

        # Menu Bar
        self.AlarmMenus = wxMenuBar()
        self.SetMenuBar(self.AlarmMenus)
        self.File = wxMenu()
        self.File.Append(ID_NEW,"&New","Create New Alarm File")
        self.File.Append(ID_OPEN,"&Open","Open Existing Alarm File")
        self.File.Append(ID_SAVE,"&Save","Save Alarm File")
        self.File.Append(ID_SAVEAS,"Save &As...","Save Alarm File As...")
        self.File.AppendSeparator()
        self.File.Append(ID_PREFS, "Preferences...", "pyAlarm Preferences")
        self.File.AppendSeparator()
        self.File.Append(ID_EXIT,"&Exit","Exit pyAlarm")
        self.AlarmMenus.Append(self.File, "File")
        self.Edit = wxMenu()
        self.Edit.Append(ID_ALARMNEW, "&New Alarm", "Create New Alarm")
        self.Edit.Append(ID_ALARMEDIT, "&Edit Alarm", "Edit Selected Alarm")
        self.Edit.Append(ID_ALARMDEL, "&Delete Alarm", "Delete Selected Alarm")
        self.AlarmMenus.Append(self.Edit, "Edit")
        self.Help = wxMenu()
        self.Help.Append(ID_ABOUT,"&About","About pyAlarm")
        self.AlarmMenus.Append(self.Help, "Help")
        self.status = self.CreateStatusBar(3)
        # Menu Bar end

        # Tool Bar
        self.toolbar = wxToolBar(self, -1)
        self.SetToolBar(self.toolbar)
        self.toolbar.SetToolBitmapSize( (32,32) )
        self.toolbar.AddSeparator()
        self.toolbar.AddLabelTool(ID_PREVIOUS, "Previous", graphics.getPreviousBitmap(), wxNullBitmap, wxITEM_NORMAL, "Previous", "Previous")
        self.toolbar.AddLabelTool(ID_NEXT, "Next", graphics.getNextBitmap(), wxNullBitmap, wxITEM_NORMAL, "Next", "Next")
        self.toolbar.AddLabelTool(ID_PLAY, "Play", graphics.getPlayBitmap(), wxNullBitmap, wxITEM_NORMAL, "Play", "Play")
        self.toolbar.AddLabelTool(ID_PAUSE, "Pause", graphics.getPauseBitmap(), wxNullBitmap, wxITEM_NORMAL, "Pause", "Pause")
        self.toolbar.AddLabelTool(ID_STOP, "Stop", graphics.getStopBitmap(), wxNullBitmap, wxITEM_NORMAL, "Stop", "Stop")
        self.toolbar.AddSeparator()
        self.toolbar.AddLabelTool(ID_SNOOZE, "Snooze", graphics.getSnoozeIconBitmap(), wxNullBitmap, wxITEM_NORMAL, "Snooze", "Snooze")
        self.toolbar.AddLabelTool(ID_SLEEP, "Go to Sleep", graphics.getGotoSleepBitmap(), wxNullBitmap, wxITEM_NORMAL, "Sleep", "Sleep")
        # Tool Bar end
        #wxBitmap("playicon.png", wxBITMAP_TYPE_ANY)
        self.__set_properties()
        self.__do_layout()
        # end wxGlade

        EVT_MENU(self, ID_NEW, self.OnNew)
        EVT_MENU(self, ID_ABOUT, self.OnAbout)
        EVT_MENU(self, ID_EXIT, self.OnExit)
        EVT_MENU(self, ID_SAVEAS, self.SaveAsAlarms)
        EVT_MENU(self, ID_SAVE, self.SaveAlarmFile)
        EVT_MENU(self, ID_OPEN, self.OpenAsAlarms)
        EVT_MENU(self, ID_PLAY, self.OnPlay)
        EVT_MENU(self, ID_PAUSE, self.OnPause)
        EVT_MENU(self, ID_STOP, self.OnStop)
        EVT_MENU(self, ID_SNOOZE, self.OnSnooze)
        EVT_MENU(self, ID_PREVIOUS, self.OnPrevious)
        EVT_MENU(self, ID_NEXT, self.OnNext)
        EVT_MENU(self, ID_ALARMNEW, self.CreateNewAlarm)
        EVT_MENU(self, ID_ALARMEDIT, self.EditAlarm)
        EVT_MENU(self, ID_ALARMDEL, self.DeleteAlarm)
        EVT_MENU(self, ID_PREFS, self.Preferences)
        EVT_MENU(self, ID_SLEEP, self.OnGotoSleep)

        EVT_CLOSE(self, self.CleanUp)
        
        paEVT_RESULT(self, self.paEventHandler)

    def __set_properties(self):
        self.SetTitle("pyAlarm")
        self.File.Enable(ID_SAVE, False)
        self.SetSize(paGlobals.gCommand.mainSize)
        self.status.SetStatusWidths([ -4,-2,-1])
        self.toolbar.Realize()
        
        # Enable/disable the tools, as needed
        self.toolbar.EnableTool(ID_PREVIOUS, False)
        self.toolbar.EnableTool(ID_NEXT, False)
        self.toolbar.EnableTool(ID_PLAY, True)
        self.toolbar.EnableTool(ID_PAUSE, False)
        self.toolbar.EnableTool(ID_STOP, False)
        self.toolbar.EnableTool(ID_SNOOZE, False)
        self.toolbar.EnableTool(ID_SLEEP, True)
        
    def __do_layout(self):
        # begin wxGlade: MainFrame.__do_layout
        self.MainNotebook.AddPage(self.AlarmPanel, "Alarm List")
        self.MainNotebook.AddPage(self.PlaylistPanel, "Playlist")
        self.MainNotebook.AddPage(self.Mixer, "Mixer")
        self.sizer = wxNotebookSizer(self.MainNotebook)
        self.SetAutoLayout(1)
        self.SetSizer(self.sizer)
        self.Layout()
        if paGlobals.gCommand.centerMe == 1:
            self.Centre()
        # end wxGlade

    def CleanUp(self, event):  # Shutting Down completely
        paGlobals.gCommand.mainSize = self.GetSizeTuple()
        paGlobals.gCommand.mainPos = self.GetPositionTuple()
        paGlobals.gCommand.CleanUp()
        self.Destroy()

    def OnNew(self, event):
        paGlobals.gCommand.NewProject()
        self.openAlarms = 0
        self.File.Enable(ID_SAVE, False)
        self.AlarmPanel.RecreateList()

    def OnAbout(self, event):
        self.dlg = paabout.paAboutWin(self, -1, 'About pyAlarm', size=(300,300))
        self.dlg.ShowModal()
        self.dlg.Destroy()

    def OnExit(self, event):
        self.Close(true)
        
    def OnPlay(self, event):
        paGlobals.gCommand.StartPlay()
        
    def OnStop(self, event):
        paGlobals.gCommand.StopPlay()
        
    def OnSnooze(self, event):
        self.toolbar.EnableTool(ID_SNOOZE, False)
        paGlobals.gCommand.SnoozePlay()
        
    def OnUnsnooze(self):
        self.toolbar.EnableTool(ID_SNOOZE, True)
    
    def OnPause(self, event):
        paGlobals.gCommand.PausePlay()
        if paGlobals.gCommand.IsPaused():
            self.toolbar.EnableTool(ID_PREVIOUS, False)
            self.toolbar.EnableTool(ID_NEXT, False)
            self.toolbar.EnableTool(ID_PLAY, False)
            self.toolbar.EnableTool(ID_PAUSE, True)
            self.toolbar.EnableTool(ID_STOP, False)
        else:
            self.toolbar.EnableTool(ID_PREVIOUS, True)
            self.toolbar.EnableTool(ID_NEXT, True)
            self.toolbar.EnableTool(ID_PLAY, False)
            self.toolbar.EnableTool(ID_PAUSE, True)
            self.toolbar.EnableTool(ID_STOP, True)
            
        
    def OnPrevious(self, event):
        paGlobals.gCommand.PreviousPlay()
        
    def OnNext(self, event):
        paGlobals.gCommand.NextPlay()

    def OnGotoSleep(self, evt):
        if paGlobals.gCommand.GotoSleep():
            self.toolbar.EnableTool(ID_SLEEP, False)
            
    def OnSoundAlarm(self):
        self.toolbar.EnableTool(ID_SNOOZE, True)

    def Preferences(self, event):
        self.preferenceDialog = preferences.paPreferences(self, -1, "Preferences")
        self.preferenceDialog.Show(true)
    
    def OpenAsAlarms(self, event):
        self.dlg = wxFileDialog(NULL, "Open...","","","*.pax",wxOPEN)
        defDir, ext = os.path.split(paGlobals.gCommand.currentFile)
        self.dlg.SetDirectory(defDir)
        if(self.dlg.ShowModal()==wxID_OK):
            self.currentFile = self.dlg.GetPath()
        else:
            self.dlg.Destroy()
            return
        self.dlg.Destroy()
        paGlobals.gCommand.OpenAlarms(self.currentFile)
        self.AlarmPanel.RecreateList()
        self.openAlarms = 1
        self.File.Enable(ID_SAVE, True)

    def SaveAsAlarms(self, event):
        self.dlg = wxFileDialog(NULL, "Save As...","","","*.pax",wxSAVE|wxOVERWRITE_PROMPT)
        defDir, ext = os.path.split(paGlobals.gCommand.currentFile)
        self.dlg.SetDirectory(defDir)
        if(self.dlg.ShowModal()==wxID_OK):
            currentFile = self.dlg.GetPath()
        else:
            self.dlg.Destroy()
            return
        self.dlg.Destroy()
        paGlobals.gCommand.SetSaveFile(currentFile)
        self.SaveAlarmFile(event)
        self.openAlarms = 1
        self.File.Enable(ID_SAVE, True)
        
    def SaveAlarmFile(self, event):
        paGlobals.gCommand.SaveAlarms()
        
    def paEventHandler(self, event):
        if event.data.dataType == paCOUNTDOWN:
            self.status.SetStatusText("-" + event.data.timeleft + " (" + event.data.alarm + ")",1)
            self.status.SetStatusText(event.data.timenow,2)
        elif event.data.dataType == paDIRSCAN:
            self.PlaylistPanel.AddFile(-1, event.data.Path)
        elif event.data.dataType == paTAGSCAN:
            self.PlaylistPanel.AddTags(event.data)
        elif event.data.dataType == paNEWSONG:
            self.OnSetCurrentFile(event.data.song)
            
    def CreateNewAlarm(self, event):
        self.AlarmPanel.CreateNewAlarm(event)
    
    def EditAlarm(self, event):
        self.AlarmPanel.EditAlarm(event)

    def DeleteAlarm(self, event):
        self.AlarmPanel.DeleteAlarm(event)

    def OnStartPlay(self):
        # Setup play buttons
        self.toolbar.EnableTool(ID_PREVIOUS, True)
        self.toolbar.EnableTool(ID_NEXT, True)
        self.toolbar.EnableTool(ID_PLAY, False)
        self.toolbar.EnableTool(ID_PAUSE, True)
        self.toolbar.EnableTool(ID_STOP, True)
        self.SetStatusBarPane(-1)
        
    def OnStopPlay(self):
        self.toolbar.EnableTool(ID_PREVIOUS, False)
        self.toolbar.EnableTool(ID_NEXT, False)
        self.toolbar.EnableTool(ID_PLAY, True)
        self.toolbar.EnableTool(ID_PAUSE, False)
        self.toolbar.EnableTool(ID_STOP, False)
        self.SetStatusBarPane(0)
        
    def OnSetCurrentFile(self, file):
        root, ext = os.path.split(file)
        self.status.SetStatusText(ext)
        
# end of class MainFrame
# Revision history
#  $Log: main.py,v $
#  Revision 1.25  2005/01/18 06:59:39  davidfancella
#  Disabling the snooze button again, on start.  Probably broke big things this time.  :)
#
#  Revision 1.24  2005/01/18 06:57:24  davidfancella
#  Basic logging implemented.
#
#  Revision 1.23  2005/01/18 06:11:42  davidfancella
#  Finally got buttons enabling/disabling, and all alarm actions going through gCommand.
#
#  Revision 1.22  2005/01/18 05:06:23  davidfancella
#  Adding logging, a little at a time
# 
