#!/usr/bin/env python

'''
File Information:
    
   Last changed by: $Author: davidfancella $
                On: $Date: 2005/01/18 06:57:22 $
                
   This file is part of $Name:  $
   
   $Revision: 1.24 $
   
'''


'''
    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

from wxPython.wx import *
from pyAlarm.data.alarms import alarmListData
from pyAlarm.data.log import paLog
from PlayListData import playListData
import action
from pyAlarm.events import *
import time, threading
from pyAlarm import pathreads
import sys, os
from pyAlarm import pluginMan

class paCommand:
    PLScanner = []
    mediaSupported = ['.mp3']
    hasTS = 0
    alarmSounding = false
    def __init__(self):
        self.SetBasePath()
        self.currentAlarmSong = ''
        self.basepath = os.path.abspath(os.path.dirname(sys.argv[0]))
        self.alarms = alarmListData()
        self.playlistdata = playListData()
        self.soundPlaying = 0
        self.LoadPreferences()
        if self.currentFile != "":
            if(self.alarms.OpenAlarmListData(self.currentFile)):
                self.openAlarms = 1
                print "Opened Alarm List"
        print "Loading plugins"
        self.plugins = pluginMan.paPluginManager(os.path.join(self.GetBasePath(), 'plugins'))
        self.plugins.LoadPlugins()
        print "Done loading plugins"
        print "Loading logger"
        self.Log = paLog(self.logFile)
        self.Log.Enable(self.logMe)
        
        self.mixer = self.plugins.GetMixer(self.prefMixer)
        self.filer = self.plugins.GetFiler(self.prefFiler)
        
        self.paTimer = pathreads.paClock()

    def LoadPreferences(self):
        self.config = wxConfig("pyAlarm","Nobody",".pyalarmrc")
        self.currentFile = self.config.Read("/Alarms/OpenFile","")
        self.snoozeminutes = self.config.ReadInt("/Alarms/SnoozeTime",9)
        self.logFile = self.config.Read("/Alarms/LogFile", os.path.expanduser("~/.pyalarmlog"))
        self.logMe = self.config.ReadInt("/Alarms/LogMe", 0)
        self.prefMixer = self.config.Read("/Plugins/Mixer","default")
        self.prefMedia = self.config.Read("/Plugins/Media","default")
        self.prefFiler = self.config.Read("/Plugins/Filer", "default")
        self.restoreInterface = self.config.ReadInt("/Interface/Restore", 1)
        if self.restoreInterface==1:
            self.mainSize = (self.config.ReadInt("/Interface/Restore/mainX", 630),self.config.ReadInt("/Interface/Restore/mainY", 443))
            self.mainPos = (self.config.ReadInt("/Interface/Restore/posX",0),self.config.ReadInt("/Interface/Restore/posY",0))
            self.centerMe = self.config.ReadInt("/Interface/Restore/centerMe",0)
        else:
            self.mainSize = (630, 443)
            self.mainPos = (0,0)
            self.centerMe = 1

    def SavePreferences(self):
        self.config.Write("/Alarms/OpenFile",self.currentFile)
        self.config.WriteInt("/Alarms/SnoozeTime",self.snoozeminutes)
        self.config.Write("/Alarms/LogFile", self.logFile)
        self.config.WriteInt("/ALarms/LogMe", self.logMe)
        self.config.Write("/Plugins/Mixer",self.prefMixer)
        self.config.Write("/Plugins/Media",self.prefMedia)
        self.config.Write("/Plugins/Filer",self.prefFiler)
        self.config.WriteInt("/Interface/Restore",self.restoreInterface)
        (x, y) = self.mainSize
        self.config.WriteInt("/Interface/Restore/mainX", x)
        self.config.WriteInt("/Interface/Restore/mainY", y)
        (x, y) = self.mainPos
        self.config.WriteInt("/Interface/Restore/posX",x)
        self.config.WriteInt("/Interface/Restore/posY",y)
        self.config.WriteInt("/Interface/Restore/centerMe",self.centerMe)
        self.config.Flush()        
        
    def GetConfig(self, config, default):
        return self.config.Read(config, default)
        
    def SetConfig(self, config, value):
        self.config.Write(config, value)
        
    def SetBasePath(self):
        if sys.platform == 'win32':
            self.__basepath = sys.prefix
        else:
            self.__basepath = os.path.join(sys.prefix,'share','pyAlarm')
    
    def GetBasePath(self):
        return self.__basepath        

    def Init(self, notify_window):
        self._notify_window = notify_window
        self.paTimer.SetAlarms(self.alarms, self._notify_window)
        print "Starting Clock"
        self.paTimer.Start(300)
        
    def NewProject(self):
        self.alarms.Clear()
        
    def CleanUp(self):
        print "Killing timer..."
        self.paTimer.Stop()
        del self.paTimer
        if self.soundPlaying == 1:
            print "Killing AudioPlayer"
            self.SoundPlayer.abort()
            print "Sleeping for one second"
            time.sleep(1)
            if callable(self.SoundPlayer):
                print "AudioPlayer still going, trying to kill it."
                del self.SoundPlayer
                if callable(self.SoundPlayer):
                    print "AudioPlayer is *still* going, you'll have to kill this program yourself."
                else:
                    print "Dead!  Finally!"
        print "Updating config object"
        self.SavePreferences()
        print "Deleting config object, should automatically write changes to itself"
        del self.config
        print "Done cleaning up command, you may start your landing now."
    
    def OpenAlarms(self,filename):
        self.NewProject()
        self.alarms.OpenAlarmListData(filename)
        self.currentFile = filename
        self.config.Write("/Alarms/OpenFile",self.currentFile)
        self.config.Flush()
        
    def SetSaveFile(self, filename):
        self.currentFile = filename
        
    def SaveAlarms(self):
        self.alarms.SaveAlarmListData(self.currentFile)
        self.config.Write("/Alarms/OpenFile",self.currentFile)
        self.config.Flush()
        
    def GotoSleep(self):
        print "Going to sleep..."
        self.Log.GotoSleep()
        return True

    def SetLogger(self):
        self.Log.Enable(self.logMe)
        self.Log.SetFilename(self.logFile)
        
    def SoundAlarm(self, alarm):
        # insert logging here
        self.alarmSounding = True
        
        # Start the alarm here, for now it's just the media player stuff
        self._notify_window.OnSoundAlarm()
        self.StartPlay()
        self.Log.Alarm(self.currentAlarmSong)
            
    def StartPlay(self):
        self._notify_window.OnStartPlay()
        if self.soundPlaying == False:
            self.SoundPlayer = action.paAction(self.plugins.GetMediaPlugins())
            self.soundPlaying = self.SoundPlayer.isPlaying
        else:
            print "Sound player already running!"
            
    def StopPlay(self):
        if self.soundPlaying == True:
            self.SoundPlayer.abort()
            self.soundPlaying = False
            self._notify_window.OnStopPlay()
            self.Log.Stop()
            if self.alarmSounding:
                self.CallWakeupConfirmation()
        else:
            print "Sound player is *not* running."
        
    def SnoozePlay(self):
        if self.soundPlaying == True:
            self.SoundPlayer.pause()
            self.snoozeTimer = threading.Timer(self.snoozeminutes*60, self.UnSnooze)
            self.snoozeTimer.start()
            self.Log.Snooze()
        
    def UnSnooze(self):
        if self.soundPlaying == True:
            self.SoundPlayer.pause()
        self._notify_window.OnUnsnooze()
        self.Log.Unsnooze()
    
    def PausePlay(self):
        if self.soundPlaying == True:
            self.SoundPlayer.pause()
        
    def IsPaused(self):
        return self.SoundPlayer.IsPaused()
            
    def NextPlay(self):
        self.SoundPlayer.next()
        
    def PreviousPlay(self):
        self.SoundPlayer.previous()
        
    def SetCurrentFile(self, file):
        eventData = paNewSong(file)
        self._notify_window.AddPendingEvent(ResultEvent(eventData))
        self.currentAlarmSong = file
                
    def CallWakeupConfirmation(self):
        # This is a placeholder, confirmation will be added here
        # For now it just silently marks the alarm as over
        self.alarmSounding = False
        self.Log.Wakeup()
        pass
        
    def FieldsFile(self, file, fields):
        pass
            
    def CurrentFile(self, file):
        pass
            
    def MediaSupported(self):
        return self.plugins.GetMediaSupported()
    
    def SavePlaylist(self, filename):
        self.playlistdata.SavePlayListData(filename)
        
    def GetPluginNames(self):
        return self.plugins.GetAllPlugins()
        
    def GetPluginCredits(self, pluginName):
        return self.plugins.GetCredits(pluginName)
        
    def GetPluginByName(self, pluginName):
        return self.plugins.GetPlugin(pluginName)
        
    def GetAllMixers(self):
        return self.plugins.GetAllMixers()
        
    def GetAllMedia(self):
        return self.plugins.GetMediaPlugins()
    
    def GetAllMediaFile(self):
        return self.plugins.GetAllFilers()
    
    def GetMixer(self):
        return self.mixer
        
    def GetFiler(self):
        return self.filer
        
    def GetAllFilers(self):
        return self.plugins.GetAllFilers()
        
    def GetAllPlugins(self):
        return self.plugins.GetAll()
        
    # The next two methods, about the FilerFile, are temporary and exist for the sake of hooking up
    # the existing GUI to the filer plugins.
    def OpenFilerFile(self, file):
        try:
            self.filer.OpenList(file)
        except:
            pass
        
    def SaveFilerFile(self, file):
        self.filer.SaveList(file)
        
    def SetPLFields(self, fields):
        pass
        
    def GivePLFields(self, fields):
        '''eventData = paTagScan(fields,0)
        event = ResultEvent(eventData)
        self._notify_window.AddPendingEvent(event)'''
        pass
        
    def InitializePL(self):
        self.filer.Initialize()
        
    # The following methods should be completely replaced when the playlist is finally abstracted
    # from the GUI far enough that generic methods will replace these
    
    def OpenPLStored(self):
        if(self.OpenFilerFile(self.config.Read("/Playlist/OpenFile","playlist"))):
            self.openPlayList = 1
            print "Opened Play List"
    
    def ScanPLDir(self, path):
        for (dirpath, dirsome, files) in os.walk(path):
            for a in files:
                self.AddPLFile(os.path.join(dirpath, a) )
                
    def GetPlaylist(self):
        return self.filer.GetList()
    
    def AddPLFile(self, path):
        self.filer.AddFile(path)
        
    def SetTags(self, tags):
        pass
        
        