#!/usr/bin/env python

'''

    @version $Id: alarms.py,v 1.5 2005/01/18 03:39:07 davidfancella Exp $

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

from wxPython.wx import *
import string
import xml.sax
from time import *
from alarmlistitem import alarmListItem
from alarmcontent import paAlarmContent
from alarmwriter import paAlarmWriter

## class: alarmListData
## This is the class that stores all of the alarms.  It stores them internally in a list and makes
## them available through a number of methods.
class alarmListData:
    ## member: alarms
    ## The internal list of alarms
    alarms = []

    ## member: todaysalarms
    ## This is a list of today's alarms.
    todaysalarms = []

    ## member: currentHighIndex
    ## An internal member that stores the current high index in the alarms list
    currentHighIndex = 0

    def __init__(self):
        # initialize the list
        pass

    def __GetNewIndex(self):
        self.currentHighIndex = self.currentHighIndex + 1
        return self.currentHighIndex

    def AddNew(self, alarm):
        alarm.index = self.__GetNewIndex()
        self.alarms.append(alarm)
        return alarm

    ## method: Modify
    def Modify(self, index, alarm):
        tmpalarm = alarm
        print index
        realindex = self.alarms.index(int(index))
        self.alarms.pop(realindex)
        tmpalarm.index = index
        self.alarms.insert(realindex,alarm)

    ## method: Delete(index)
    def Delete(self, index):
        realindex = self.alarms.index(int(index))
        self.alarms.pop(realindex)

    ## method: ReadItem(index)
    ## Return the alarm stored at index, which you will no pass in.
    def GetItem(self, index):
        print index
        itemindex = self.alarms.index(int(index))
        return self.alarms[itemindex]

    ## method: Count
    ## Returns the number of alarms in the list
    def Count(self):
        return len(self.alarms)

    ## method: Clear
    ## Clear the list without deleting the alarmList object
    def Clear(self):
        self.alarms = []

        ## method: OpenAlarmListData(filename)
        ## Pass it filename, which must be a csv file.  This method will open the file and load its
        ## data into the class's data structures
    def OpenAlarmListData(self,filename):
        #try:
        contentParser = paAlarmContent()
        xml.sax.parse(filename,contentParser)
        self.Clear()
        for alarm in contentParser.alarms:
            self.AddNew(alarm)
        #except:
        #    print "Failed to open " + filename
    
    ## method: SaveAlarmListData(filename)
    def SaveAlarmListData(self,filename):
        contentWriter = paAlarmWriter()
        contentWriter.Write(filename,self.alarms)
    
	## method: GetNextAlarm(timenow)
	## Pass it timenow (wxDateTime) and this method will return the very next alarm that is scheduled to go off
    def GetNextAlarm(self,timenow):
        # We're gonna do this a dumb way
        tmpalarms = []
        
        for ala in self.alarms:
            tmpInt = ala.TimeToNext(timenow)
            if tmpInt is not None:
                tmpalarms.append(tmpInt)
            
        tmpalarms.sort()
        tmpalarms.reverse()
        if len(tmpalarms) > 0:
            nextInt = tmpalarms.pop()
        else:
            return None
        
        for ala in self.alarms:
            if ala.TimeToNext(timenow) == nextInt:
                return ala
        
