#!/usr/bin/env python

'''

    @version $Id: plugin.py,v 1.14 2005/01/14 00:52:37 davidfancella Exp $

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

import imp, string, sys
import random

paMEDIA = 'media'
paDEFAULT = 'default'
paALARM = 'alarm'
paMIXER = 'mixer'
paALL = 'all'
paFILE = 'file'

# pluginBroke exists so we can force subclasses of paPlugin to define methods.  paPlugin
# will throw a pluginBroke exception in any method that *must* be overridden.

class pluginBroke(Exception):
    def __init__(self, arg):
        self.arg = arg

    def __str__(self):
        return repr(self.arg)

# paPlugin is the master plugin class from which all others must derive

class pluginData:
    def __init__(self):
        pass
        
    

class paPlugin:
    wantsFile = False
    pluginType = paDEFAULT
    IsPlaying = False
    __thisString = 'pyAlarm Plugin Not Specified'
    __modules = dict()
    __IsBroke = False
    __FailedDeps = dict()
    __platforms = dict()
    __credits = dict()
    __options = dict()
    __callbacks = dict()
        
    def __init__(self):
        self.ID = self.GetID()
    
    def __str__(self):
        return self.Name()
              
    def Initialize(self):
        pass
        
    def GetID(self):
        num = random.randint(0,10000)
        self.__modules[str(num)] = dict()
        self.__FailedDeps[str(num)] = []
        self.__platforms[str(num)] = []
        self.__callbacks[str(num)] = dict()
        self.__options[str(num)] = dict()
        self.__credits[str(num)] = dict()
        self.__credits[str(num)]['author'] = ''
        self.__credits[str(num)]['copyright'] = ''
        self.__credits[str(num)]['version'] = ''
        self.__credits[str(num)]['license'] = ''
        self.__credits[str(num)]['url'] = ''
        return num
        
    def GetCredits(self):
        return self.__credits[str(self.ID)]

    def GetOptionValue(self, option):
        (optiontype, defaultvalue, defaultvalue, comment, choices) = self.__options[str(self.ID)][option]
        return self.Callback('GetConfig')(self, option, defaultvalue)
        
    def SetOption(self, option, optiontype, defaultvalue, comment, choices = []):
        self.__options[str(self.ID)][option] = (optiontype, defaultvalue, defaultvalue, comment, choices)
        
    def GetOption(self, option):
        return self.__options[str(self.ID)][option]
        
    def GetOptionNames(self):
        return self.__options[str(self.ID)].keys()
                
    def SetAuthor(self, author):
        self.__credits[str(self.ID)]['author'] = author
        
    def SetCopyright(self, copyr):
        self.__credits[str(self.ID)]['copyright'] = copyr
        
    def SetVersion(self, version):
        self.__credits[str(self.ID)]['version'] = version
        
    def SetLicense(self, licen):
        self.__credits[str(self.ID)]['license'] = licen
        
    def SetAuthUrl(self, url):
        self.__credits[str(self.ID)]['url'] = url
        
    def Break(self):
        self.__IsBroke = True
        
    def IsBroken(self):
        return self.__IsBroke
        
    def GetFailedDeps(self):
        return self.__FailedDeps[str(self.ID)]
        
    # do not override aSetup, it's used internally for platform management and dependency tracking.
    def aSetup(self):
        if sys.platform not in self.__platforms[str(self.ID)]:
             self.__IsBroke = True
    
    def SetCallback(self, callback, func):
        self.__callbacks[callback] = func
        
    def Callback(self, callback):
        return self.__callbacks[callback]
       
    def AddPlatform(self, platform):
        if platform == 'all':
            self.__platforms[str(self.ID)].append(sys.platform)
        else:
            self.__platforms[str(self.ID)].append(platform)
            
    def GetPlatforms(self):
        return self.__platforms[str(self.ID)]
            
    def MyNameIs(self, name):
        self.__thisString = name
        
    def Name(self):
        return self.__thisString
    
    def GetModules(self):
        return self.__modules[str(self.ID)]
        
    def LoadModule(self, module):
        try:
            self.__modules[str(self.ID)][module] = __import__(module)
            
            #newmodule = string.split(module,'.')
            #newpath = None
            #for mod in newmodule:
            #    (f, p, d) = imp.find_module(mod,newpath)
            #    self.__modules[str(self.ID)][mod] = imp.load_module(mod,f,p,d)
            #    try:
            #        newpath = self.__modules[str(self.ID)][mod].__path__
            #    except AttributeError:
            #        pass  # ignore it if it's the last item
            #        
            return True
        except ImportError:
            self.__IsBroke = True
            self.__FailedDeps[str(self.ID)].append(module)
            return False
            
            
            
            
            
            
        