#!/usr/bin/env python

'''
File Information:
    
   Last changed by: $Author: davidfancella $
                On: $Date: 2005/01/18 07:03:48 $
                
   This file is part of $Name:  $
   
   $Revision: 1.14 $
   
'''


'''

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

import paGlobals
import plugin.plugin
import sys, os, imp

class paPluginManager:
    allPlugins = []
    mixerPlugins = []
    brokePlugins = []
    def __init__(self, plugindir):
        self.plugindir = plugindir
        self.platform = sys.platform
        
    def LoadPlugins(self):
        plugins = os.listdir(self.plugindir)
        sys.path.append(sys.prefix)
        sys.path.append(self.plugindir)
        for tplugin in plugins:
            pluginPath = os.path.join(self.plugindir, tplugin)
            if os.path.isfile(pluginPath):
                roota, exta = os.path.split(pluginPath)
                root, ext = os.path.splitext(exta)
                pluginLibpath = os.path.join(roota,'lib' + root)
                sys.path.append(pluginLibpath)
                if root != 'setup':  # for working in CVS code where a setup.py script resides in plugins
                    if ext == '.py':
                        #execfile(pluginPath,globals(),locals())
                        exec('import ' + root)
                        tmpExec = root + '.' + root + '()\n'
                        tmpPlugin = eval(tmpExec)
                        tmpPlugin.Setup()
                        tmpPlugin.aSetup()
                        if tmpPlugin.IsBroken():
                            self.brokePlugins.append(tmpPlugin)
                            print '----------------------------------------'
                            print
                            print tmpPlugin.Name() + ' is broke'
                            print
                            print 'Loaded dependencies:'
                            for dep in tmpPlugin.GetModules().keys():
                                print '     ' + dep
                            print
                            print 'Failed dependencies:'
                            for dep in tmpPlugin.GetFailedDeps():
                                print '     ' + dep
                            print
                        elif tmpPlugin.pluginType == plugin.plugin.paMEDIA:
                            self.allPlugins.append(tmpPlugin)
                            print tmpPlugin.Name() + " supports:"
                            for a in tmpPlugin.GetSupports():
                                print '        --- ' + a
                            tmpPlugin.SetCallback('SetFile', self.PluginSetFileCallback)
                        elif tmpPlugin.pluginType == plugin.plugin.paMIXER:
                            self.allPlugins.append(tmpPlugin)
                            print '----------------------------------------'
                            print tmpPlugin.Name() + " is a Mixer"
                            print '----------------------------------------'
                        elif tmpPlugin.pluginType == plugin.plugin.paFILE:
                            self.allPlugins.append(tmpPlugin)
                            print '----------------------------------------'
                            print tmpPlugin.Name() + " is a File plugin"
                            print '----------------------------------------'
                            tmpPlugin.SetCallback('GiveFields', self.PluginGiveFieldsCallback)
                            tmpPlugin.SetCallback('SetFields', self.PluginSetFieldsCallback)
                        # set callbacks
                        tmpPlugin.SetCallback('GetConfig', self.PluginGetConfigCallback)

    def PluginGetConfigCallback(self, plugin, config, defaultvalue):
        pluginConfigpath = "/Plugins/" + plugin.Name() + config
        return paGlobals.gCommand.GetConfig(pluginConfigpath, defaultvalue)
        
    def PluginGiveFieldsCallback(self, plugin, fields = []):
        paGlobals.gCommand.GivePLFields(fields)

    def PluginSetFieldsCallback(self, plugin, fields = []):
        paGlobals.gCommand.SetPLFields(fields)
        
    def PluginSetFileCallback(self, plugin, file):
        paGlobals.gCommand.SetCurrentFile(file)
                                    
    def GetMediaSupported(self):
        tmpSupp = []
        for mediaplugin in self.allPlugins:
            if mediaplugin.pluginType == plugin.plugin.paMEDIA:
                tmpSupp2 = mediaplugin.GetSupports()
                for tmpS in tmpSupp2:
                    tmpSupp.append(tmpS)
        return tmpSupp
        
    def GetMediaPlugins(self):
        tmpNames = []
        for mediaplugin in self.allPlugins:
            if mediaplugin.pluginType == plugin.plugin.paMEDIA:
                tmpNames.append(mediaplugin)
        if len(tmpNames) > 0:
            return tmpNames
        else:
            return None
            
    def GetAllPlugins(self):
        tmpNames = []
        for mediaplugin in self.allPlugins:
            tmpNames.append(mediaplugin.Name())
        for mixerplugin in self.brokePlugins:
            tmpNames.append(mixerplugin.Name())
        return tmpNames
        
    def GetAll(self):
        tmpNames = []
        for mediaplugin in self.allPlugins:
            tmpNames.append(mediaplugin)
        for mixerplugin in self.brokePlugins:
            tmpNames.append(mixerplugin)
        return tmpNames
        
    def GetPlugin(self, pluginName):
        for mixerplugin in self.allPlugins:
            if mixerplugin.Name() == pluginName:
                return mixerplugin
        return None
        
    def GetCredits(self, thisplugin):
        for mediaplugin in self.allPlugins:
            if mediaplugin.Name() == thisplugin:
                return mediaplugin.GetCredits()
        for mediaplugin in self.brokePlugins:
            if mediaplugin.Name() == thisplugin:
                return mediaplugin.GetCredits()
        return None
        
    def GetAllMixers(self):
        tmpName = []
        for mixerplugin in self.allPlugins:
            if mixerplugin.pluginType == plugin.plugin.paMIXER:
                tmpName.append(mixerplugin)
        return tmpName    
        
    def GetMixer(self, mixer="default"):
        tmpName = []
        try:
            for mixerplugin in self.allPlugins:
                if mixerplugin.pluginType == plugin.plugin.paMIXER:
                    tmpName.append(mixerplugin)
            if mixer == "default":
                return tmpName[0]
            else:
                for mixerplugin in tmpName:
                    if mixerplugin.Name() == mixer:
                        return mixerplugin
                
        except IndexError:
            return None
            
    def GetAllFilers(self):
        tmpName = []
        for mixerplugin in self.allPlugins:
            if mixerplugin.pluginType == plugin.plugin.paFILE:
                tmpName.append(mixerplugin)
        return tmpName    
        
    def GetFiler(self, filer="default"):
        tmpName = []
        try:
            for mixerplugin in self.allPlugins:
                if mixerplugin.pluginType == plugin.plugin.paFILE:
                    tmpName.append(mixerplugin)
            if filer == "default":
                return tmpName[0]
            else:
                for mixerplugin in tmpName:
                    if mixerplugin.Name() == filer:
                        return mixerplugin
                
        except IndexError:
            return None
        

