#!/usr/bin/env python

'''

    @version $Id: paPlaylist.py,v 1.7 2005/01/18 03:39:07 davidfancella Exp $

    This file is part of pyAlarm.

    pyAlarm is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    pyAlarm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pyAlarm; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	pyAlarm is copyright 2003 by David Fancella

'''

from pyAlarm.plugin.fileplugin import *

class paPlaylist(paFilePlugin):
    __playlist = []
    def __init__(self):
        paFilePlugin.__init__(self)
    
    def Setup(self):
        self.LoadModule("os")
        self.LoadModule("random")
        self.LoadModule("copy")
        self.LoadModule("ID3")
        self.LoadModule("string")
        self.AddPlatform("linux2")
        self.AddPlatform("win32")
        
        self.AddSupport("mp3")
        self.AddSupport("ogg")
        self.AddSupport("wav")
        self.AddSupport("flac")

        self.AddField("Artist")
        self.AddField("Album")
                
        self.MyNameIs('Media Playlist')
            
        self.SetAuthor('Dave Fancella')
        self.SetCopyright('2004')
        self.SetLicense('GNU General Public License v2.0 or higher')
        self.SetAuthUrl('http://www.davefancella.com/')
        self.SetVersion('0.1')
        
        self.SetOption('Randomize', 'yesno', 'True', "Randomize playlist")
        
        self.modules = self.GetModules()
        
    def Initialize(self):
        self.Callback('SetFields')(self, ['Artist', 'Album', 'Song'])
        
    def AddFile(self, file):
        file = self.modules["string"].strip(file)
        self.__playlist.append(file)
        idTags = self.modules['ID3'].ID3(file)
        self.Callback('GiveFields')(self, { 'Artist' : idTags.artist,
                                            'Album'  : idTags.album,
                                            'Song'   : idTags.title
                                          })
        
    def RenderList(self):
        self.songqueue = self.modules["copy"].deepcopy(self.__playlist)
        if self.GetOptionValue('Randomize') == 'True':
            if(len(self.songqueue) > 0):
                # randomize songqueue - should probably be done somewhere else
                nCards = len(self.songqueue)
                if(nCards > 1):
                    for i in range(nCards):
                        j = self.modules["random"].randrange(i, nCards)
                        [self.songqueue[i], self.songqueue[j]] = [self.songqueue[j], self.songqueue[i]]
        self.listPointer = -1
                    
    def GetNextFile(self):
        self.listPointer += 1
        if(self.listPointer >= len(self.songqueue)):
            self.listPointer = 0
        return self.songqueue[self.listPointer]
        
    def GetPreviousFile(self):
        self.listPointer -= 1
        if(self.listPointer < 0):
            self.listPointer = len(self.songqueue) - 1
        return self.songqueue[self.listPointer]
                                
    def GetList(self):
        return self.__playlist
        
    def ClearList(self):
        del self.__playlist[0:len(self.__playlist)]
    
    def OpenList(self, file):
        fp = open(file,"r")
        
        for line in fp:
            self.AddFile(line)
        fp.close()
        
    def SaveList(self, file):
        fp = open(file,"w")
        
        for line in self.__playlist:
            fp.write(line + "\n")
            
        fp.close()
        
            
            

